import { useCallback, useEffect, useRef } from "react";
import { useFrame } from "@react-three/fiber";
import { usePlayButtonStore, usePauseButtonStore, useResetButtonStore } from "../../../../../store/usePlayButtonStore";
import { useMaterialStore } from "../../../../../store/simulation/useMaterialStore";

interface DelayInstance {
    delayEndTime: number;
    materialId?: string;
    action: ConveyorAction;
    isPaused: boolean;
    remainingTime: number;
}

export function useDelayHandler() {
    const { isPlaying } = usePlayButtonStore();
    const { isPaused } = usePauseButtonStore();
    const { isReset } = useResetButtonStore();
    const { setIsPaused } = useMaterialStore();
    const activeDelays = useRef<Map<string, DelayInstance>>(new Map());

    const cleanupDelay = useCallback(() => {
        activeDelays.current.clear();
    }, []);

    useEffect(() => {
        if (isReset) {
            cleanupDelay();
        }
    }, [isReset, cleanupDelay]);

    const delayLogStatus = (materialUuid: string, status: string) => {
        // console.log(`${materialUuid}, ${status}`);
    }

    useEffect(() => {
        const currentTime = performance.now();

        activeDelays.current.forEach((delay) => {
            if (isPaused && !delay.isPaused) {
                delay.remainingTime = Math.max(0, delay.delayEndTime - currentTime);
                delay.isPaused = true;
            } else if (!isPaused && delay.isPaused) {
                delay.delayEndTime = currentTime + delay.remainingTime;
                delay.isPaused = false;
                delay.remainingTime = 0;
            }
        });
    }, [isPaused]);

    useFrame(() => {
        if (!isPlaying || isPaused) return;

        const currentTime = performance.now();
        const completedDelays: string[] = [];

        activeDelays.current.forEach((delay, key) => {
            if (!delay.isPaused && currentTime >= delay.delayEndTime && delay.materialId) {
                delayLogStatus(delay.materialId, `Delay completed}`);
                setIsPaused(delay.materialId, false);
                completedDelays.push(key);
            }
        });

        completedDelays.forEach(key => {
            activeDelays.current.delete(key);
        });
    });

    const handleDelay = useCallback((action: ConveyorAction, materialId?: string) => {
        if (!action || action.actionType !== 'delay' || !isPlaying || !materialId) return;

        const delayMs = (action.delay || 0) * 1000;
        if (delayMs <= 0) return;

        const key = materialId ? `${materialId}-${action.actionUuid}` : action.actionUuid;

        // If this material already has a delay, cancel it
        if (activeDelays.current.has(key)) {
            activeDelays.current.delete(key);
        }

        activeDelays.current.set(key, {
            delayEndTime: performance.now() + delayMs,
            materialId,
            action,
            isPaused: false,
            remainingTime: 0
        });

        delayLogStatus(materialId, `Started ${delayMs * 1000}s delay`);
        setIsPaused(materialId, true);

    }, [isPlaying]);

    useEffect(() => {
        return () => {
            cleanupDelay();
        };
    }, [cleanupDelay]);

    return {
        handleDelay,
        cleanupDelay
    };
}